@@ -3,12 +3,12 @@
  * varchar.c
  *	  Functions for the built-in types char(n) and varchar(n).
  *
- * Portions Copyright (c) 1996-2003, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2005, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /cvsroot/pgsql-server/src/backend/utils/adt/varchar.c,v 1.102 2003/08/04 04:03:10 tgl Exp $
+ *	  $PostgreSQL: pgsql/src/backend/utils/adt/varchar.c,v 1.113.2.2 2006/05/21 20:05:48 tgl Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -16,6 +16,8 @@
 
 #include "access/hash.h"
 #include "catalog/pg_type.h"
+#include "lib/stringinfo.h"
+#include "libpq/pqformat.h"
 #include "miscadmin.h"
 #include "utils/array.h"
 #include "utils/builtins.h"
@@ -36,8 +38,8 @@
  * length also. (eg. in INSERTs, we have the tupleDescriptor which contains
  * the length of the attributes and hence the exact length of the char() or
  * varchar(). We pass this to bpcharin() or varcharin().) In the case where
- * we cannot determine the length, we pass in -1 instead and the input string
- * must be null-terminated.
+ * we cannot determine the length, we pass in -1 instead and the input
+ * converter does not enforce any length check.
  *
  * We actually implement this as a varlena so that we don't have to pass in
  * the length for the comparison functions. (The difference between these
@@ -52,12 +54,85 @@
  *	 bpchar - char()														 *
  *****************************************************************************/
 
+/*
+ * bpchar_input -- common guts of bpcharin and bpcharrecv
+ *
+ * s is the input text of length len (may not be null-terminated)
+ * atttypmod is the typmod value to apply
+ *
+ * Note that atttypmod is measured in characters, which
+ * is not necessarily the same as the number of bytes.
+ *
+ * If the input string is too long, raise an error, unless the extra
+ * characters are spaces, in which case they're truncated.  (per SQL)
+ */
+static BpChar *
+bpchar_input(const char *s, size_t len, int32 atttypmod)
+{
+	BpChar	   *result;
+	char	   *r;
+	size_t		maxlen;
+
+	/* If typmod is -1 (or invalid), use the actual string length */
+	if (atttypmod < (int32) VARHDRSZ)
+		maxlen = len;
+	else
+	{
+		size_t		charlen;	/* number of CHARACTERS in the input */
+
+		maxlen = atttypmod - VARHDRSZ;
+		charlen = pg_mbstrlen_with_len(s, len);
+		if (charlen > maxlen)
+		{
+			/* Verify that extra characters are spaces, and clip them off */
+			size_t		mbmaxlen = pg_mbcharcliplen(s, len, maxlen);
+			size_t		j;
+
+			/*
+			 * at this point, len is the actual BYTE length of the input
+			 * string, maxlen is the max number of CHARACTERS allowed for this
+			 * bpchar type, mbmaxlen is the length in BYTES of those chars.
+			 */
+			for (j = mbmaxlen; j < len; j++)
+			{
+				if (s[j] != ' ')
+					ereport(ERROR,
+							(errcode(ERRCODE_STRING_DATA_RIGHT_TRUNCATION),
+							 errmsg("value too long for type character(%d)",
+									(int) maxlen)));
+			}
+
+			/*
+			 * Now we set maxlen to the necessary byte length, not the number
+			 * of CHARACTERS!
+			 */
+			maxlen = len = mbmaxlen;
+		}
+		else
+		{
+			/*
+			 * Now we set maxlen to the necessary byte length, not the number
+			 * of CHARACTERS!
+			 */
+			maxlen = len + (maxlen - charlen);
+		}
+	}
+
+	result = (BpChar *) palloc(maxlen + VARHDRSZ);
+	VARATT_SIZEP(result) = maxlen + VARHDRSZ;
+	r = VARDATA(result);
+	memcpy(r, s, len);
+
+	/* blank pad the string if necessary */
+	if (maxlen > len)
+		memset(r + len, ' ', maxlen - len);
+
+	return result;
+}
+
 /*
  * Convert a C string to CHARACTER internal representation.  atttypmod
  * is the declared length of the type plus VARHDRSZ.
- *
- * If the C string is too long, raise an error, unless the extra
- * characters are spaces, in which case they're truncated.  (per SQL)
  */
 Datum
 bpcharin(PG_FUNCTION_ARGS)
@@ -69,67 +144,8 @@ bpcharin(PG_FUNCTION_ARGS)
 #endif
 	int32		atttypmod = PG_GETARG_INT32(2);
 	BpChar	   *result;
-	char	   *r;
-	size_t		len,
-				maxlen;
-	int			i;
-	int			charlen;		/* number of charcters in the input string */
-
-	/* verify encoding */
-	len = strlen(s);
-	pg_verifymbstr(s, len, false);
-
-	charlen = pg_mbstrlen(s);
-
-	/* If typmod is -1 (or invalid), use the actual string length */
-	if (atttypmod < (int32) VARHDRSZ)
-		maxlen = charlen;
-	else
-		maxlen = atttypmod - VARHDRSZ;
-
-	if (charlen > maxlen)
-	{
-		/* Verify that extra characters are spaces, and clip them off */
-		size_t		mbmaxlen = pg_mbcharcliplen(s, len, maxlen);
-
-		/*
-		 * at this point, len is the actual BYTE length of the input
-		 * string, maxlen is the max number of CHARACTERS allowed for this
-		 * bpchar type.
-		 */
-		if (strspn(s + mbmaxlen, " ") == len - mbmaxlen)
-			len = mbmaxlen;
-		else
-			ereport(ERROR,
-					(errcode(ERRCODE_STRING_DATA_LENGTH_MISMATCH),
-					 errmsg("value too long for type character(%d)",
-							(int) maxlen)));
-
-		/*
-		 * XXX: at this point, maxlen is the necessary byte length, not
-		 * the number of CHARACTERS!
-		 */
-		maxlen = len;
-	}
-	else
-	{
-		/*
-		 * XXX: at this point, maxlen is the necessary byte length, not
-		 * the number of CHARACTERS!
-		 */
-		maxlen = len + (maxlen - charlen);
-	}
-
-	result = palloc(maxlen + VARHDRSZ);
-	VARATT_SIZEP(result) = maxlen + VARHDRSZ;
-	r = VARDATA(result);
-	for (i = 0; i < len; i++)
-		*r++ = *s++;
-
-	/* blank pad the string if necessary */
-	for (; i < maxlen; i++)
-		*r++ = ' ';
 
+	result = bpchar_input(s, strlen(s), atttypmod);
 	PG_RETURN_BPCHAR_P(result);
 }
 
@@ -159,8 +175,20 @@ bpcharout(PG_FUNCTION_ARGS)
 Datum
 bpcharrecv(PG_FUNCTION_ARGS)
 {
-	/* Exactly the same as textrecv, so share code */
-	return textrecv(fcinfo);
+	StringInfo	buf = (StringInfo) PG_GETARG_POINTER(0);
+
+#ifdef NOT_USED
+	Oid			typelem = PG_GETARG_OID(1);
+#endif
+	int32		atttypmod = PG_GETARG_INT32(2);
+	BpChar	   *result;
+	char	   *str;
+	int			nbytes;
+
+	str = pq_getmsgtext(buf, buf->len - buf->cursor, &nbytes);
+	result = bpchar_input(str, nbytes, atttypmod);
+	pfree(str);
+	PG_RETURN_BPCHAR_P(result);
 }
 
 /*
@@ -197,15 +225,19 @@ bpchar(PG_FUNCTION_ARGS)
 	char	   *r;
 	char	   *s;
 	int			i;
-	int			charlen;		/* number of charcters in the input string
-								 * + VARHDRSZ */
+	int			charlen;		/* number of characters in the input string +
+								 * VARHDRSZ */
+
+	/* No work if typmod is invalid */
+	if (maxlen < (int32) VARHDRSZ)
+		PG_RETURN_BPCHAR_P(source);
 
 	len = VARSIZE(source);
 
 	charlen = pg_mbstrlen_with_len(VARDATA(source), len - VARHDRSZ) + VARHDRSZ;
 
-	/* No work if typmod is invalid or supplied data matches it already */
-	if (maxlen < (int32) VARHDRSZ || charlen == maxlen)
+	/* No work if supplied data matches typmod already */
+	if (charlen == maxlen)
 		PG_RETURN_BPCHAR_P(source);
 
 	if (charlen > maxlen)
@@ -221,24 +253,24 @@ bpchar(PG_FUNCTION_ARGS)
 			for (i = maxmblen - VARHDRSZ; i < len - VARHDRSZ; i++)
 				if (*(VARDATA(source) + i) != ' ')
 					ereport(ERROR,
-							(errcode(ERRCODE_STRING_DATA_LENGTH_MISMATCH),
-						  errmsg("value too long for type character(%d)",
-								 maxlen - VARHDRSZ)));
+							(errcode(ERRCODE_STRING_DATA_RIGHT_TRUNCATION),
+							 errmsg("value too long for type character(%d)",
+									maxlen - VARHDRSZ)));
 		}
 
 		len = maxmblen;
 
 		/*
-		 * XXX: at this point, maxlen is the necessary byte
-		 * length+VARHDRSZ, not the number of CHARACTERS!
+		 * XXX: at this point, maxlen is the necessary byte length+VARHDRSZ,
+		 * not the number of CHARACTERS!
 		 */
 		maxlen = len;
 	}
 	else
 	{
 		/*
-		 * XXX: at this point, maxlen is the necessary byte
-		 * length+VARHDRSZ, not the number of CHARACTERS!
+		 * XXX: at this point, maxlen is the necessary byte length+VARHDRSZ,
+		 * not the number of CHARACTERS!
 		 */
 		maxlen = len + (maxlen - charlen);
 	}
@@ -249,12 +281,11 @@ bpchar(PG_FUNCTION_ARGS)
 	VARATT_SIZEP(result) = maxlen;
 	r = VARDATA(result);
 
-	for (i = 0; i < len - VARHDRSZ; i++)
-		*r++ = *s++;
+	memcpy(r, s, len - VARHDRSZ);
 
 	/* blank pad the string if necessary */
-	for (; i < maxlen - VARHDRSZ; i++)
-		*r++ = ' ';
+	if (maxlen > len)
+		memset(r + len - VARHDRSZ, ' ', maxlen - len);
 
 	PG_RETURN_BPCHAR_P(result);
 }
@@ -342,14 +373,53 @@ name_bpchar(PG_FUNCTION_ARGS)
  *****************************************************************************/
 
 /*
- * Convert a C string to VARCHAR internal representation.  atttypmod
- * is the declared length of the type plus VARHDRSZ.
+ * varchar_input -- common guts of varcharin and varcharrecv
  *
- * Note that atttypmod is regarded as the number of characters, which
+ * s is the input text of length len (may not be null-terminated)
+ * atttypmod is the typmod value to apply
+ *
+ * Note that atttypmod is measured in characters, which
  * is not necessarily the same as the number of bytes.
  *
- * If the C string is too long, raise an error, unless the extra characters
- * are spaces, in which case they're truncated.  (per SQL)
+ * If the input string is too long, raise an error, unless the extra
+ * characters are spaces, in which case they're truncated.  (per SQL)
+ */
+static VarChar *
+varchar_input(const char *s, size_t len, int32 atttypmod)
+{
+	VarChar    *result;
+	size_t		maxlen;
+
+	maxlen = atttypmod - VARHDRSZ;
+
+	if (atttypmod >= (int32) VARHDRSZ && len > maxlen)
+	{
+		/* Verify that extra characters are spaces, and clip them off */
+		size_t		mbmaxlen = pg_mbcharcliplen(s, len, maxlen);
+		size_t		j;
+
+		for (j = mbmaxlen; j < len; j++)
+		{
+			if (s[j] != ' ')
+				ereport(ERROR,
+						(errcode(ERRCODE_STRING_DATA_RIGHT_TRUNCATION),
+					  errmsg("value too long for type character varying(%d)",
+							 (int) maxlen)));
+		}
+
+		len = mbmaxlen;
+	}
+
+	result = (VarChar *) palloc(len + VARHDRSZ);
+	VARATT_SIZEP(result) = len + VARHDRSZ;
+	memcpy(VARDATA(result), s, len);
+
+	return result;
+}
+
+/*
+ * Convert a C string to VARCHAR internal representation.  atttypmod
+ * is the declared length of the type plus VARHDRSZ.
  */
 Datum
 varcharin(PG_FUNCTION_ARGS)
@@ -361,33 +431,8 @@ varcharin(PG_FUNCTION_ARGS)
 #endif
 	int32		atttypmod = PG_GETARG_INT32(2);
 	VarChar    *result;
-	size_t		len,
-				maxlen;
-
-	/* verify encoding */
-	len = strlen(s);
-	pg_verifymbstr(s, len, false);
-
-	maxlen = atttypmod - VARHDRSZ;
-
-	if (atttypmod >= (int32) VARHDRSZ && len > maxlen)
-	{
-		/* Verify that extra characters are spaces, and clip them off */
-		size_t		mbmaxlen = pg_mbcharcliplen(s, len, maxlen);
-
-		if (strspn(s + mbmaxlen, " ") == len - mbmaxlen)
-			len = mbmaxlen;
-		else
-			ereport(ERROR,
-					(errcode(ERRCODE_STRING_DATA_LENGTH_MISMATCH),
-				  errmsg("value too long for type character varying(%d)",
-						 (int) maxlen)));
-	}
-
-	result = palloc(len + VARHDRSZ);
-	VARATT_SIZEP(result) = len + VARHDRSZ;
-	memcpy(VARDATA(result), s, len);
 
+	result = varchar_input(s, strlen(s), atttypmod);
 	PG_RETURN_VARCHAR_P(result);
 }
 
@@ -417,8 +462,20 @@ varcharout(PG_FUNCTION_ARGS)
 Datum
 varcharrecv(PG_FUNCTION_ARGS)
 {
-	/* Exactly the same as textrecv, so share code */
-	return textrecv(fcinfo);
+	StringInfo	buf = (StringInfo) PG_GETARG_POINTER(0);
+
+#ifdef NOT_USED
+	Oid			typelem = PG_GETARG_OID(1);
+#endif
+	int32		atttypmod = PG_GETARG_INT32(2);
+	VarChar    *result;
+	char	   *str;
+	int			nbytes;
+
+	str = pq_getmsgtext(buf, buf->len - buf->cursor, &nbytes);
+	result = varchar_input(str, nbytes, atttypmod);
+	pfree(str);
+	PG_RETURN_VARCHAR_P(result);
 }
 
 /*
@@ -471,9 +528,9 @@ varchar(PG_FUNCTION_ARGS)
 		for (i = maxmblen; i < len - VARHDRSZ; i++)
 			if (*(VARDATA(source) + i) != ' ')
 				ereport(ERROR,
-						(errcode(ERRCODE_STRING_DATA_LENGTH_MISMATCH),
-				  errmsg("value too long for type character varying(%d)",
-						 maxlen - VARHDRSZ)));
+						(errcode(ERRCODE_STRING_DATA_RIGHT_TRUNCATION),
+					  errmsg("value too long for type character varying(%d)",
+							 maxlen - VARHDRSZ)));
 	}
 
 	len = maxmblen + VARHDRSZ;
@@ -510,14 +567,16 @@ Datum
 bpcharlen(PG_FUNCTION_ARGS)
 {
 	BpChar	   *arg = PG_GETARG_BPCHAR_P(0);
+	int			len;
 
-	/* optimization for single byte encoding */
-	if (pg_database_encoding_max_length() <= 1)
-		PG_RETURN_INT32(VARSIZE(arg) - VARHDRSZ);
+	/* get number of bytes, ignoring trailing spaces */
+	len = bcTruelen(arg);
 
-	PG_RETURN_INT32(
-			  pg_mbstrlen_with_len(VARDATA(arg), VARSIZE(arg) - VARHDRSZ)
-		);
+	/* in multibyte encoding, convert to number of characters */
+	if (pg_database_encoding_max_length() != 1)
+		len = pg_mbstrlen_with_len(VARDATA(arg), len);
+
+	PG_RETURN_INT32(len);
 }
 
 Datum
@@ -549,11 +608,14 @@ bpchareq(PG_FUNCTION_ARGS)
 	len1 = bcTruelen(arg1);
 	len2 = bcTruelen(arg2);
 
-	/* fast path for different-length inputs */
+	/*
+	 * Since we only care about equality or not-equality, we can avoid all
+	 * the expense of strcoll() here, and just do bitwise comparison.
+	 */
 	if (len1 != len2)
 		result = false;
 	else
-		result = (varstr_cmp(VARDATA(arg1), len1, VARDATA(arg2), len2) == 0);
+		result = (strncmp(VARDATA(arg1), VARDATA(arg2), len1) == 0);
 
 	PG_FREE_IF_COPY(arg1, 0);
 	PG_FREE_IF_COPY(arg2, 1);
@@ -573,11 +635,14 @@ bpcharne(PG_FUNCTION_ARGS)
 	len1 = bcTruelen(arg1);
 	len2 = bcTruelen(arg2);
 
-	/* fast path for different-length inputs */
+	/*
+	 * Since we only care about equality or not-equality, we can avoid all
+	 * the expense of strcoll() here, and just do bitwise comparison.
+	 */
 	if (len1 != len2)
 		result = true;
 	else
-		result = (varstr_cmp(VARDATA(arg1), len1, VARDATA(arg2), len2) != 0);
+		result = (strncmp(VARDATA(arg1), VARDATA(arg2), len1) != 0);
 
 	PG_FREE_IF_COPY(arg1, 0);
 	PG_FREE_IF_COPY(arg2, 1);
@@ -685,12 +750,48 @@ bpcharcmp(PG_FUNCTION_ARGS)
 	PG_RETURN_INT32(cmp);
 }
 
+Datum
+bpchar_larger(PG_FUNCTION_ARGS)
+{
+	BpChar	   *arg1 = PG_GETARG_BPCHAR_P(0);
+	BpChar	   *arg2 = PG_GETARG_BPCHAR_P(1);
+	int			len1,
+				len2;
+	int			cmp;
+
+	len1 = bcTruelen(arg1);
+	len2 = bcTruelen(arg2);
+
+	cmp = varstr_cmp(VARDATA(arg1), len1, VARDATA(arg2), len2);
+
+	PG_RETURN_BPCHAR_P((cmp >= 0) ? arg1 : arg2);
+}
+
+Datum
+bpchar_smaller(PG_FUNCTION_ARGS)
+{
+	BpChar	   *arg1 = PG_GETARG_BPCHAR_P(0);
+	BpChar	   *arg2 = PG_GETARG_BPCHAR_P(1);
+	int			len1,
+				len2;
+	int			cmp;
+
+	len1 = bcTruelen(arg1);
+	len2 = bcTruelen(arg2);
+
+	cmp = varstr_cmp(VARDATA(arg1), len1, VARDATA(arg2), len2);
+
+	PG_RETURN_BPCHAR_P((cmp <= 0) ? arg1 : arg2);
+}
+
 
 /*
  * bpchar needs a specialized hash function because we want to ignore
  * trailing blanks in comparisons.
  *
- * XXX is there any need for locale-specific behavior here?
+ * Note: currently there is no need for locale-specific behavior here,
+ * but if we ever change the semantics of bpchar comparison to trust
+ * strcoll() completely, we'd need to do something different in non-C locales.
  */
 Datum
 hashbpchar(PG_FUNCTION_ARGS)
